<?php 
namespace PortalPeople\magento\controllers\api;

use Yii;
use yii\rest\Controller;
use yii\filters\VerbFilter;
use PortalPeople\magento\models\Carts;
use yii\web\Request;

class OrdersController extends Controller
{

    public function actionNewOrder() {
        /** @var Request $request */
        $request = Yii::$app->request;
        return Carts::NewMagentoOrder($request->getBodyParams());
    }

}