<?php
namespace PortalPeople\magento\models\carts;
use yii;
use yii\helpers\Json;

class Magento extends \common\models\DynamicModel
{

    public static function segment_title()
    {
        return 'Magento Attributes';
    }
    /**
     * Undocumented function
     *
     *    'colour' => [
     *        'rules' => [
     *            'string'
     *        ],
     *        'label' => Yii::t('app', "Colour",),
     *        'input' => 'default',
     *     ],
     * @return void
     */
    public static function localattributes()
    {
        return [
            'orderid' => [
                'rules' => [
                    ['integer']
                ],
                'label' => Yii::t('app', "Order ID",),
                'input' => 'default',
            ],
            'shipping_date' => [
                'rules' => [
                    ['string']
                ],
                'label' => Yii::t('app', "Shipping Date",),
                'input' => 'default',
            ],
            'shipping_w3w' => [
                'rules' => [
                    ['string']
                ],
                'label' => Yii::t('app', "Shipping What3Words",),
                'input' => 'default',
            ],
            'shipping_message' => [
                'rules' => [
                    ['string']
                ],
                'label' => Yii::t('app', "Shipping Message",),
                'input' => 'default',
            ],
        ];
    }


    public function dataLoad($modelData) {
    }
}